sealed class EventsEvent {
  const EventsEvent();
}

class EventsInitialEvent extends EventsEvent {
  const EventsInitialEvent();
}

class EventsTabChangedEvent extends EventsEvent {
  final int tabIndex;

  const EventsTabChangedEvent({required this.tabIndex});
}

class EventsSubTabChangedEvent extends EventsEvent {
  final int subTabIndex;

  const EventsSubTabChangedEvent({required this.subTabIndex});
}

class LoadTabEventEvent extends EventsEvent {
  final int tabIndex;
  final int subTabIndex;

  const LoadTabEventEvent({required this.tabIndex, required this.subTabIndex});
}

class LoadMoreEventEvent extends EventsEvent {
  final int tabIndex;
  final int subTabIndex;

  const LoadMoreEventEvent({required this.tabIndex, required this.subTabIndex});
}
